/* main.qml  -- Main Qml program for Scriptomatix */
/*
   Copyright (C) 2007-2018 Bruno ANSELME <be.root@free.fr>

   This file is part of the Scriptomatix software.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License v3 for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.4
import QtQuick.Controls 1.3
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import QtQuick.Layouts 1.1
import QtQuick.Controls.Styles 1.3
import Qt.labs.settings 1.0
import QmlFile 0.1
import QmlEnvVars 0.1

ApplicationWindow {
    id: scriptomatix
    property string sx_app_name: "Scriptomatix"
    property string sx_groupname: ""
    property string sx_scriptpath: ""
    property string sx_scriptargs: ""
    property bool sx_exitRequired: false
    property bool sx_timed: false
    property int  sx_batchnum: -1
    property int  sx_dirnum: -1
    property int sx_cellWidth: 120
    property int sx_cellHeight: 65
    property int sx_nbLineDir: 1
    property int sx_nbColumnScript: 2

    function pad(val)      { return (val>9) ? val.toString() : "0"+val.toString(); }
    function toTime(delai) { return pad(Math.floor(delai / 3600))+":"+pad(Math.floor((delai % 3600) / 60))+":"+pad(Math.floor((delai % 3600) % 60)); }

    function updateHelp(item) {
        if (item.helptext !== "") {
            sx_helpmesg.text = item.helptext;
            sx_helpmesg.height = sx_helpmesg.contentHeight+3
            sx_helpmesg.y = item.y + item.height + 3;
            sx_helpmesg.x = item.x + 3;
            sx_helpmesg.visible = true;
            sx_hideTimer.restart();
        }
    }

    function startScript() {
        sx_batchnum++;
        if (sx_batchnum >= sx_menuscript.sx_scriptview.model.count) {
            sx_batchnum = -1;
        } else {
            sx_menuscript.sx_scriptview.currentIndex = sx_batchnum;
            sx_stdio.output.addText("----- "+sx_stdio.cmd.text+" -----\n");
            sx_stdio.error.addText("----- "+sx_stdio.cmd.text+" -----\n");
            sx_execAction.trigger();
        }
    }

    function scriptChanged(sname) {
        sx_menuscript.ready = false;
        sx_menuscript.sx_scriptview.model.clear();
        sx_menuscript.sx_scriptview.delegate = sx_menuscript.sx_scriptview.emptyscriptdelegate;
        sx_io.source = sx_groupname = sname;
        sx_io.read();
        sx_scriptpath = sx_groupname.replace(/\/?[^\/]*$/,'');   // Delete file name
        try {
            sx_menuscript.sx_scriptview.model = Qt.createQmlObject(sx_io.text, sx_menuscript.sx_scriptview, sx_io.source);
            sx_menuscript.sx_scriptview.delegate = sx_menuscript.sx_scriptview.scriptdelegate;
            sx_stdio.shell = sx_menuscript.sx_scriptview.model.shell;
        } catch(error) {
            sx_stdio.clear();
            sx_stdio.error.addText(error);
            sx_parameters.width   = 0;
        }
        sx_menuscript.ready = true;
    }

    function toModelSource(item)
    {
        var str = "import QtQuick 2.4  \n";
        str += "ListModel {\n";
        for (var i=0; i<item.count; i++) {
            str += "  ListElement {\n";
            var p = item.get(i);
            str += "    path:  \""+p.path+"\"\n";
            str += "    title: \""+p.title+"\"\n";
            str += "    icon:  \""+p.icon+"\"\n";
            str += "  }\n";
        }
        str += "}\n"
        return str;
    }

    function saveDirmodel() {
        sx_io.setSource(sx_environment.getVar("HOME")+"/.Scripts.qml");
        sx_io.setText(toModelSource(sx_menudir.sx_dirview.model));
        sx_io.write();
    }

    Settings {      // Persistent parameters
        property alias sx_width: scriptomatix.width
        property alias sx_height: scriptomatix.height
        property alias sx_dirnum: scriptomatix.sx_dirnum
        property alias sx_menustate: sx_menudir.state
        property alias sx_nbLineDir: scriptomatix.sx_nbLineDir
        property alias sx_nbColumnScript: scriptomatix.sx_nbColumnScript
    }

    SystemPalette { id: sx_palette; colorGroup: SystemPalette.Active }

    title: sx_app_name+" - "+sx_environment.getVar("USER")+"@"+sx_environment.getHostName();
    minimumWidth: 620
    minimumHeight: 300
    width: 1024
    height: 480
    visible: true
    color: sx_palette.window

    Action {
        id: sx_execAction
        shortcut: "Ctrl+Return"
        iconSource: "file:/"+dataPath+"icons/process-run.png"
        tooltip: qsTr("Run script (%1)").arg(shortcut);
        onTriggered: {
            sx_stdio.start(sx_scriptpath, (sx_batchnum === -1), sx_timed);
            sx_execAction.enabled = false;
            sx_stopAction.enabled = true;
            sx_killAction.enabled = true;
            sx_clearAction.enabled = false;
            sx_runallAction.enabled = false;
            sx_menuscript.enabled = false;
            sx_menudir.sx_dirview.enabled = false;
            sx_parameters.enabled = false;
            sx_butdir.enabled = false;
            sx_butadddir.enabled = false;
            sx_butdeldir.enabled = false;
            sx_buttimed.enabled = false;
        }
    }
    Action {
        id: sx_stopAction
        shortcut: "Ctrl+S"
        iconSource: "file:/"+dataPath+"icons/process-stop.png"
        enabled: false
        tooltip: qsTr("Stop process : SIGTERM (%1)").arg(shortcut);
        onTriggered: if (sx_stdio.running) { sx_stdio.terminate(); }
    }
    Action {
        id: sx_killAction
        shortcut: "Ctrl+K"
        iconSource: "file:/"+dataPath+"icons/process-kill.png"
        enabled: false
        tooltip: qsTr("Kill process : SIGKILL (%1)").arg(shortcut);
        onTriggered: if (sx_stdio.running) { sx_stdio.kill(); }
    }
    Action {
        id: sx_clearAction
        shortcut: "Ctrl+F"
        iconSource: "file:/"+dataPath+"icons/edit-clear.png"
        tooltip: qsTr("Clear outputs (%1)").arg(shortcut);
        onTriggered: if (!sx_stdio.running) { sx_stdio.clear(); }
    }
    Action {
        id: sx_runallAction
        shortcut: "Ctrl+A"
        iconSource: "file:/"+dataPath+"icons/run-all.png"
        tooltip: qsTr("Run all scripts starting from selected script (%1)").arg(shortcut);
        enabled: sx_menuscript.sx_scriptview.model.batch
        onTriggered: {
            if (sx_menuscript.sx_scriptview.currentIndex != -1)
                sx_batchnum = sx_menuscript.sx_scriptview.currentIndex-1;
            else
                sx_batchnum = -1;
            sx_stdio.clear();
            startScript();
        }
    }
    Action {
        id: sx_quitAction
        shortcut: "Ctrl+Q"
        iconSource: "file:/"+dataPath+"icons/application-exit.png"
        tooltip: qsTr("Quit (%1)").arg(shortcut);
        onTriggered: {
            if (sx_stdio.running) {
                sx_exitRequired = true;
                sx_confirmExit.visible = true;
            } else {
                Qt.quit();
            }
        }
    }
    toolBar: ToolBar {
        height: 36
        RowLayout {
            ToolButton { action: sx_execAction }
            ToolButton { action: sx_runallAction; enabled: sx_menuscript.sx_scriptview.model.batch }
            ToolButton { action: sx_stopAction }
            ToolButton { action: sx_killAction }
            Separator {}
            ToolButton { action: sx_clearAction }
            Text { text: toTime(sx_stdio.processtime) }
            Separator {}
            ToolButton {
                id: sx_buttimed
                iconSource: "file:/"+dataPath+"icons/chronometer.png"
                tooltip: qsTr("System time");
                checkable: true
                checked: sx_timed;
                onClicked: sx_timed = !sx_timed;
            }
            Separator { width: 30; color: "transparent" }
            ToolButton {
                id: sx_butdir
                iconSource: "file:/"+dataPath+"icons/folder-activities.png"
                tooltip: qsTr("Scripts sets")
                checkable: true
                checked: ( sx_menudir.state === "opened") ? true : false;
                onClicked: sx_menudir.state = ( sx_menudir.state === "opened") ? "" : "opened";
            }
            ToolButton {
                id: sx_butadddir
                iconSource: "file:/"+dataPath+"icons/list-add.png"
                tooltip: qsTr("Add a script set")
                enabled: editable;
                visible: ( sx_menudir.state !== "")
                onClicked: {
                    sx_fileDialog.folder = scriptomatix.sx_scriptpath;
                    sx_fileDialog.open();
                }
            }
            ToolButton {
                id: sx_butdeldir
                iconSource: "file:/"+dataPath+"icons/list-remove.png"
                tooltip: qsTr("Remove current script set")
                enabled: editable;
                visible: ( sx_menudir.state !== "")
                onClicked: sx_confirmDelete.visible = true;
            }
        }
        RowLayout {
            anchors.right: parent.right
            Text {
                text: sx_menuscript.sx_scriptview.model.title;
                font.pixelSize: 14
            }
            ToolButton {
                id: sx_butdev
                iconSource: "file:/"+dataPath+"icons/view-source.png"
                tooltip: qsTr("View sources")
                onClicked: sx_source.state = (sx_help.state === "opened") ? "" : "opened";
            }
            Separator {}
            ToolButton {
                id: sx_buthelp
                iconSource: "file:/"+dataPath+"icons/help-about.png"
                tooltip: qsTr("Help")
                onClicked: sx_help.state = (sx_help.state === "opened") ? "" : "opened";
            }
            ToolButton { action: sx_quitAction; id: sx_butquit }
        }
    }
    SplitView  {
        id: vsplit
        orientation: Qt.Vertical
        anchors.fill: parent
        anchors.margins: 5
        DirView { id: sx_menudir }
        Item {
            id: sx_mainContent
            Layout.fillWidth: true
            Layout.fillHeight: true
            anchors.margins: 3
            SplitView  {
                id: hsplit
                orientation: Qt.Horizontal
                anchors.fill: parent
                ScriptView {id: sx_menuscript }
                Item {
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    Item {
                        id: sx_parameters
                        property variant pannel: null
                        anchors.top: parent.top
                        anchors.bottom: parent.bottom
                        anchors.left: parent.left
                        width: 0
                        anchors.margins: 3
                        opacity: (enabled) ? 1.0 : 0.5
                        Rectangle {
                            anchors.fill: parent
                            radius: 5.0
                            color: sx_palette.alternateBase     // SystemPalette type
                            border.color: sx_palette.mid
                            border.width: 1
                        }
                        Behavior on width {
                            enabled: sx_stdio.output.lineCount < 5000;
                            NumberAnimation { duration: 200 }
                        }
                        Keys.onReturnPressed: {
                            if (event.modifiers & Qt.ControlModifier)
                                sx_execAction.trigger();
                        }
                    }
                    Stdio {
                        id: sx_stdio
                        anchors.top: parent.top
                        anchors.bottom: parent.bottom
                        anchors.left: sx_parameters.right
                        width: parent.width - sx_parameters.width
                        anchors.right: parent.right
                        anchors.margins: 3
                        onEnded: {
                            sx_execAction.enabled = true;
                            sx_stopAction.enabled = false;
                            sx_killAction.enabled = false;
                            sx_clearAction.enabled = true;
                            sx_runallAction.enabled = sx_menuscript.sx_scriptview.model.batch;
                            sx_menuscript.enabled = true;
                            sx_menudir.sx_dirview.enabled = true;
                            sx_parameters.enabled = true;
                            sx_butdir.enabled = true;
                            sx_butadddir.enabled = true;
                            sx_butdeldir.enabled = true;
                            sx_buttimed.enabled = true;
                            if (sx_exitRequired) {
                                Qt.quit();
                            }
                            if (sx_batchnum != -1) startScript();
                        }
                    }
                }
            }
            TextArea {
                id: sx_helpmesg
                textFormat: TextEdit.RichText
                style: TextAreaStyle {
                    backgroundColor: sx_palette.window
                }
                anchors.margins: 5
                anchors.leftMargin: 15
                text: ""
                textColor: sx_palette.text
                visible: false
                readOnly: true
                opacity: 0.7
                Timer {
                    id: sx_hideTimer
                    interval: 4000
                    onTriggered: parent.visible = false;
                }
            }
        } // Item sx_mainContent
    } // vsplit

    FileIO     { id: sx_io }
    ShellEnv   { id: sx_environment }
    Help       { id: sx_help }
    Source     { id: sx_source }

    BusyIndicator {
        running: sx_stdio.running
        anchors.centerIn: parent
    }

    FileDialog {
        id: sx_fileDialog
        title: qsTr("Choose a script file")
        nameFilters: [ "Qml file (Scripts.qml)" ]
        modality: Qt.WindowModal
        onAccepted: {
            var str = sx_fileDialog.fileUrl.toString();
            str = str.replace(/file:\/\//,'');                     // Delete protocol
            scriptChanged(str);
            sx_menudir.sx_dirview.model.append({ path: sx_scriptpath, title: sx_menuscript.sx_scriptview.model.title, icon: sx_menuscript.sx_scriptview.model.scripticon})
            sx_menudir.sx_dirview.currentIndex = sx_menudir.sx_dirview.count - 1;
            saveDirmodel();
            sx_menuscript.sx_scriptview.delegate = sx_menuscript.sx_scriptview.scriptdelegate;
        }
    }

    MessageDialog {
        id: sx_confirmDelete
        width: 400
        height: 100
        visible: false
        title: qsTr("Confirmation")
        text: qsTr("Do you want to remove script folder ? : %1").arg(sx_menuscript.sx_scriptview.model.title);
        standardButtons: StandardButton.Cancel | StandardButton.Ok
        onAccepted: {
            sx_dirnum = sx_menudir.sx_dirview.currentIndex;
            sx_menudir.sx_dirview.model.remove( sx_menudir.sx_dirview.currentIndex);
            saveDirmodel();
            if (sx_dirnum > 0) sx_dirnum--;
            sx_menudir.sx_dirview.currentIndex = sx_dirnum;
        }
    }

    MessageDialog {
        id: sx_confirmExit
        width: 400
        height: 100
        visible: false
        title: qsTr("Confirmation")
        text: qsTr("A script is running !\nWould you interrupt ?")
        standardButtons: StandardButton.Cancel | StandardButton.Ok
        onAccepted: sx_stdio.kill();
        onRejected: sx_exitRequired = false;
    }

    onSx_scriptargsChanged: {
        sx_stdio.cmd.text = sx_menuscript.sx_scriptview.currentItem.cmd+" "+sx_scriptargs;
    }
    Component.onCompleted: {
        sx_io.setSource(sx_environment.getVar("HOME")+"/.Scripts.qml");
        if (sx_io.exists()) {
            sx_io.read();
            try {
                sx_menudir.sx_dirview.model = Qt.createQmlObject(sx_io.text, sx_menudir.sx_dirview, sx_io.source);
            } catch (error) {
                sx_stdio.clear();
                sx_stdio.error.addText(error);
            }
        } else {    // First use, add examples scripts set
            scriptChanged("/usr/share/scriptomatix/examples/Scripts.qml");
            sx_menudir.sx_dirview.model.append({ path: sx_scriptpath, title: sx_menuscript.sx_scriptview.model.title, icon: sx_menuscript.sx_scriptview.model.scripticon})
            saveDirmodel();
            sx_menuscript.sx_scriptview.delegate = sx_menuscript.sx_scriptview.scriptdelegate;
        }
        sx_menudir.ready = true;
        sx_menudir.height = scriptomatix.sx_nbLineDir*scriptomatix.sx_cellHeight;
        if (sx_dirnum != -1) sx_menudir.sx_dirview.currentIndex = sx_dirnum;
    }
}
